body {
  font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Helvetica,
  Arial, sans-serif;
  margin: auto;
}

@font-face {
  font-family: 'RosarioRegular';
  src: url("../assets/fonts/Rosario-Regular.ttf") format("truetype");
  font-weight: normal;
  font-style: normal;
}