@tailwind base;
@tailwind components;
@tailwind utilities;

@font-face {
    font-family: 'RosarioRegular';
    src: url('/public/fonts/Rosario-Regular.ttf') format('truetype');
}

header {
    color: black;
}

header span {
    color: blue;
}

body {
    font-family: RosarioRegular, sans-serif;
    margin: 0;
    padding: 0;
    width: 100%;
    height: 100vh;
    display: flex;
    flex-direction: column;
    align-items: center;
    justify-content: center;
    text-align: center;
}

footer {
    color: black;
}

.logo-size {
    width: 20px;
    height: 20px;
}